function Game() {
  
  this.WIDTH = 960;
  this.HEIGHT = 600;
  
  this.DEBUG = false;
  
  this.TIME_PER_FRAME = 33;
  
  this.interval;
  
  this.state = -1;
  this.nextState = -1;
  
  this.loadingScreen;
  this.loginScreen;
  
  this.shaking;
  
  this.roomInfoBox;

  
  this.init = function() {
    this.drawEmpty(); 
    performanceMonitor.init();
    preloadingManager.preloadLoadingScreen();
  };
  
  
  this.showLoadingScreen = function() {
    this.setState(Game.LOADING_SCREEN_STATE);
    this.startLoop();
  };
  
  
  this.setState = function(state) {
    this.nextState = state;
  };
  
  
  this.startLoop = function() {
    this.interval = window.setInterval(function() {
      game.loop();
    }, this.TIME_PER_FRAME);
  };
  
  
  this.loop = function() {
    this.initState();
    this.update();
    this.draw();
    performanceMonitor.update();
  };
  
  
  this.initState = function() {
    if(this.state != this.nextState) {
      this.state = this.nextState;
      
      if(this.state == Game.LOADING_SCREEN_STATE) {
        this.loadingScreen = new LoadingScreen();
        preloadingManager.preloadGame();
        
      } else if(this.state == Game.LOGIN_SCREEN_STATE) {
        this.loginScreen = new LoginScreen();
        this.loginScreen.init();
        
      } else if(this.state == Game.INGAME_STATE) {
        this.setupGame(); 
      }
    }
  };
  
  
  this.setupGame = function() {
    
    keyboard.init();
    mouse.init();
    
    keyboard.registerKeyUpHandler(Keyboard.P, function() {
      game.paused = !game.paused;
    });
    
    keyboard.registerKeyUpHandler(Keyboard.M, function() {
      sound.muteUnmute();
    });
    
    this.shaking = new Shaking();
    
    map = new Map();
    map.init();
    
    player = new Player();
    player.init();
    
    map.placeStartingCrates();
    
    this.roomInfoBox = new RoomInfoBox();
    this.roomInfoBox.init();
    
    hint = new Hint();
    hint.init();
    
    buildScreen = new BuildScreen();
    buildScreen.init();
    
    sound.play("music");
    
  };
  
  
  this.update = function() {
    
    if(this.state == Game.LOADING_SCREEN_STATE) {
      this.loadingScreen.update();

    } else if(this.state == Game.INGAME_STATE) {
      
      player.update();
      map.update();
      
      if(player.hasBunker) {
        bunker.update();
      }
      
      this.roomInfoBox.update();
      
    } 
  };
  
  
  this.draw = function() {
    
    this.drawEmpty();
    
    if(this.state == Game.LOADING_SCREEN_STATE) {
      this.loadingScreen.draw();
      
    } else if(this.state == Game.LOGIN_SCREEN_STATE) {
      this.loginScreen.draw();
      
    } else if(this.state == Game.INGAME_STATE) {
      
      c.fillStyle = "#fff";
      c.fillRect(0, 0, this.WIDTH, this.HEIGHT);
      
      this.shaking.apply();
      
      map.draw();

      this.shaking.remove();
      
      if(player.hasBunker && player.onBunker == player.myBunker && player.y > -10) {
        bunker.drawInfoBox();
      }
      
      player.drawInfoBox();
      
      this.roomInfoBox.draw();
      
      buildScreen.draw();
      
      hint.setText("this is a test text with words and so on.", 1);
      hint.draw();
      
      player.drawDeath();
    }
  };
  
  
  this.drawEmpty = function() {
    c.fillStyle = "#000";
    c.fillRect(0, 0, this.WIDTH, this.HEIGHT);
  };

}


Game.LOADING_SCREEN_STATE = 0;
Game.LOGIN_SCREEN_STATE = 1;
Game.INGAME_STATE = 2;